<?php
/**
 * Скрипт для анализа IP-адресов из логов iskwaf2x и заполнения таблицы iskwaf2x_ip2location_details (MODX2)
 */

define('MODX_API_MODE', true);

$rootPath = dirname(__FILE__, 6);
if (!file_exists($rootPath . '/config.core.php')) {
    die('Could not find MODX config.core.php. Adjust the path if necessary. Current base path assumed: ' . $rootPath);
}

require_once $rootPath . '/config.core.php';
require_once MODX_CORE_PATH . 'model/modx/modx.class.php';

$modx = new modX();
$modx->initialize('web');


if (php_sapi_name() !== 'cli' && MODX_API_MODE !== true) {
    die('Access denied.');
}

$corePath = $modx->getOption('core_path', null, MODX_CORE_PATH);
$componentName = 'iskwaf2x';


$ip2locationLibFound = false;
if (!$ip2locationLibFound) {
    $ip2locationLibSrcPath = $corePath . 'components/' . $componentName . '/lib/ip2location/src/';
    $ip2locationFilesToInclude = ['IpTools.php', 'Country.php', 'Region.php', 'Database.php'];
    $allFilesIncluded = true;
    foreach ($ip2locationFilesToInclude as $file) {
        $filePath = $ip2locationLibSrcPath . $file;
        if (file_exists($filePath)) {
            require_once $filePath;
        } else {
            if (php_sapi_name() === 'cli') fwrite(STDERR, "[iskwaf2x IP Analyzer] IP2Location library file not found: " . $filePath . "\n");
            $allFilesIncluded = false;
        }
    }
    if ($allFilesIncluded && class_exists('IP2Location\Database')) {
        $ip2locationLibFound = true;
    }
}

if (!$ip2locationLibFound) {
    if (php_sapi_name() === 'cli') fwrite(STDERR, "[iskwaf2x IP Analyzer] IP2Location PHP library could not be loaded. Please check paths or use Composer.\n");
    die();
}


$modelPath = $corePath . 'components/' . $componentName . '/model/';
if (!$modx->addPackage('iskwaf2x', $modelPath)) {
    if (php_sapi_name() === 'cli') fwrite(STDERR, "[iskwaf2x IP Analyzer] Could not load iskwaf2x model package.\n");
    die();
}


$db11Path = $corePath . 'components/' . $componentName . '/geoip_db/IP2LOCATION-LITE-DB11.BIN';
$dbAsnPath = $corePath . 'components/' . $componentName . '/geoip_db/IP2LOCATION-LITE-ASN.BIN';

$db11Reader = null;
$dbAsnReader = null;

if (file_exists($db11Path)) {
    try {
        $db11Reader = new \IP2Location\Database($db11Path, \IP2Location\Database::FILE_IO);
    } catch (\Exception $e) {
        if (php_sapi_name() === 'cli') fwrite(STDERR, "[iskwaf2x IP Analyzer] Error initializing IP2Location DB11 Reader: " . $e->getMessage() . "\n");
    }
} else {
    if (php_sapi_name() === 'cli') fwrite(STDERR, "[iskwaf2x IP Analyzer] IP2Location DB11.BIN file not found at: " . $db11Path . "\n");
}

if (file_exists($dbAsnPath)) {
    try {
        $dbAsnReader = new \IP2Location\Database($dbAsnPath, \IP2Location\Database::FILE_IO);
    } catch (\Exception $e) {
        if (php_sapi_name() === 'cli') fwrite(STDERR, "[iskwaf2x IP Analyzer] Error initializing IP2Location ASN DB Reader: " . $e->getMessage() . "\n");
    }
} else {
    if (php_sapi_name() === 'cli') fwrite(STDERR, "[iskwaf2x IP Analyzer] IP2Location ASN.BIN file not found at: " . $dbAsnPath . "\n");
}

$startTime = microtime(true);
if (php_sapi_name() === 'cli') echo "--- Starting IP Address Details Population (IP2Location) ---\n";

$limitPerRun = $modx->getOption('iskwaf2x_iskwaf2x_ip_analysis_limit_per_run', null, 500);
$processedCount = 0;
$updateIntervalDays = (int) $modx->getOption('iskwaf2x_iskwaf2x_ip_analysis_update_interval', null, 30);


$cLog = $modx->newQuery('IskWafLog');
$cLog->select('DISTINCT IskWafLog.ip_address');
$cLog->leftJoin('IskWafIp2LocationDetail', 'IpDetail', 'IskWafLog.ip_address = IpDetail.ip_address');
$cLog->where([
    'IpDetail.ip_address IS NULL',
    'OR:IpDetail.last_checked:<' => date('Y-m-d H:i:s', strtotime('-' . $updateIntervalDays . ' days')),
    'OR:IpDetail.last_checked:IS' => null
]);
$cLog->limit($limitPerRun);

$tableLog = $modx->getTableName('IskWafLog');
$tableIpDetail = $modx->getTableName('IskWafIp2LocationDetail');
$sql = "
    SELECT DISTINCT l.ip_address
    FROM {$tableLog} l
    LEFT JOIN {$tableIpDetail} d ON l.ip_address = d.ip_address
    WHERE d.ip_address IS NULL
       OR d.last_checked < '" . date('Y-m-d H:i:s', strtotime('-' . $updateIntervalDays . ' days')) . "'
       OR d.last_checked IS NULL
    LIMIT " . intval($limitPerRun);

$stmt = $modx->query($sql);
$uniqueIpsToProcess = [];
if ($stmt) {
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
        if (!empty($row['ip_address'])) {
            $uniqueIpsToProcess[] = $row['ip_address'];
        }
    }
}
if (php_sapi_name() === 'cli') echo "Found " . count($uniqueIpsToProcess) . " unique IP(s) to process.\n";

foreach ($uniqueIpsToProcess as $ip) {
    if (empty($ip) || !filter_var($ip, FILTER_VALIDATE_IP)) {
        if (php_sapi_name() === 'cli') echo "Skipping invalid IP: {$ip}\n";
        continue;
    }

    /** @var xPDOObject $ipDetailObject */
    $ipDetailObject = $modx->getObject('IskWafIp2LocationDetail', ['ip_address' => $ip]);
    if (!$ipDetailObject) {
        $ipDetailObject = $modx->newObject('IskWafIp2LocationDetail');
        $ipDetailObject->set('ip_address', $ip);
    }

    $ipDataUpdated = false;

    if ($db11Reader) {
        try {
            $geoRecords = $db11Reader->lookup($ip, \IP2Location\Database::ALL);
            if ($geoRecords && (!isset($geoRecords['countryCode']) || $geoRecords['countryCode'] !== '-')) {
                $ipDetailObject->set('ip_from', $geoRecords['ipFrom'] ?? null);
                $ipDetailObject->set('ip_to', $geoRecords['ipTo'] ?? null);
                $ipDetailObject->set('country_code', $geoRecords['countryCode'] ?? null);
                $ipDetailObject->set('country_name', $geoRecords['countryName'] ?? null);
                $ipDetailObject->set('region_name', $geoRecords['regionName'] ?? null);
                $ipDetailObject->set('city_name', $geoRecords['cityName'] ?? null);
                $ipDetailObject->set('latitude', $geoRecords['latitude'] ?? null);
                $ipDetailObject->set('longitude', $geoRecords['longitude'] ?? null);
                $ipDetailObject->set('zip_code', $geoRecords['zipCode'] ?? null);
                $ipDetailObject->set('time_zone', $geoRecords['timeZone'] ?? null);
                $ipDataUpdated = true;
            } elseif ($ipDetailObject->isNew() && php_sapi_name() === 'cli') {
                echo "IP {$ip} not found or no detailed Geo data in DB11 LITE.\n";
            }
        } catch (\Exception $e) {
            if (php_sapi_name() === 'cli') fwrite(STDERR, "[iskwaf2x IP Analyzer] Error during DB11 lookup for IP {$ip}: " . $e->getMessage() . "\n");
        }
    }

    if ($dbAsnReader) {
        try {
            $asnRecords = $dbAsnReader->lookup($ip, \IP2Location\Database::ALL);
            if ($asnRecords && isset($asnRecords['asn']) && $asnRecords['asn'] != '-' && !empty($asnRecords['asn'])) {
                $asnNumber = filter_var($asnRecords['asn'], FILTER_SANITIZE_NUMBER_INT);
                $asName = $asnRecords['as'] ?? null;
                $asName = mb_substr($asName,0, 190); 

                if ($asnNumber) {
                    $ipDetailObject->set('asn', 'AS' . $asnNumber);
                } else {
                    $ipDetailObject->set('asn', null);
                }

                $ipDetailObject->set('as_name', ($asName && $asName !== '-') ? $asName : null);
                $ipDetailObject->set('cidr', $asnRecords['cidr'] ?? null);
                $ipDataUpdated = true;
            } elseif ($ipDetailObject->isNew() && php_sapi_name() === 'cli') {
                echo "ASN data for IP {$ip} not found in ASN DB.\n";
            }
        } catch (\Exception $e) {
            if (php_sapi_name() === 'cli') fwrite(STDERR, "[iskwaf2x IP Analyzer] Error during ASN DB lookup for IP {$ip}: " . $e->getMessage() . "\n");
        }
    }

    if ($ipDataUpdated || $ipDetailObject->isNew()) {
        $ipDetailObject->set('last_checked', date('Y-m-d H:i:s'));
        if ($ipDetailObject->save()) {
            $processedCount++;
        } else {
            if (php_sapi_name() === 'cli') fwrite(STDERR, "[iskwaf2x IP Analyzer] Error saving IP detail for {$ip}.\n");
        }
    }
}

$endTime = microtime(true);
$executionTime = round($endTime - $startTime, 4);

if (php_sapi_name() === 'cli') {
    echo "Processed/updated details for {$processedCount} IP(s).\n";
    echo "IP Address Details Population finished in " . $executionTime . " seconds.\n";
}
exit(0);
