<?php
/**
 * Закачка баз гео-локации для iskwaf2x (MODX2)
 */

define('MODX_API_MODE', true);
$modxBasePath = dirname(__FILE__, 6);
if (!file_exists($modxBasePath . '/index.php')) {
    output_message('Error: MODX index.php not found. Path: ' . $modxBasePath, true);
    exit(1);
}
require_once $modxBasePath . '/index.php';


global $modx;

if (!$modx || !($modx instanceof modX)) {
    output_message('Error: MODX object not initialized.', true);
    exit(1);
}


if (php_sapi_name() !== 'cli' && MODX_API_MODE !== true) {
    output_message('Access denied. This script is intended for CLI execution or MODX API mode.', true);
    exit(1);
}


$corePath = $modx->getOption('core_path', null, MODX_CORE_PATH);
$componentName = 'iskwaf2x';
$downloadTokenSettingKey = 'iskwaf2x_iskwaf2x_ip2location_download_token';


$tempDir = $corePath . 'components/' . $componentName . '/tmp_db_download/';
$finalDbDir = $corePath . 'components/' . $componentName . '/geoip_db/';


$databasesToDownload = [
    'DB11LITEBIN'  => 'IP2LOCATION-LITE-DB11.BIN', 
    'DBASNLITEBIN' => 'IP2LOCATION-LITE-ASN.BIN',  
];


$fileMappings = [
    'DB11LITEBIN' => [
        'zip_filename_on_server' => 'IP2LOCATION-LITE-DB11.BIN.ZIP',
        'bin_in_zip'             => 'IP2LOCATION-LITE-DB11.BIN'
    ],
    'DBASNLITEBIN' => [
        'zip_filename_on_server' => 'IP2LOCATION-LITE-ASN.BIN.ZIP',
        'bin_in_zip'             => 'IP2LOCATION-LITE-ASN.BIN'
    ]
];


function output_message($message, $isError = false) {
    global $modx;
    $timestamp = date('Y-m-d H:i:s');
    if (php_sapi_name() === 'cli') {
        if ($isError) {
            fwrite(STDERR, "{$timestamp} [ERROR] {$message}\n");
        } else {
            echo "{$timestamp} [INFO] {$message}\n";
        }
    } else {
        if ($modx && $modx instanceof modX) {
            $level = $isError ? modX::LOG_LEVEL_ERROR : modX::LOG_LEVEL_INFO;
            $modx->log($level, "[iskwaf2x DB Downloader] " . $message);
        }
    }
}


output_message("Starting IP2Location LITE database download process...");


$token = $modx->getOption($downloadTokenSettingKey, null, '');
if (empty($token)) {
    output_message("Download token '{$downloadTokenSettingKey}' is not set in MODX system settings.", true);
    exit(1);
}
output_message("Using download token from MODX settings.");


if (!is_dir($tempDir)) {
    if (!@mkdir($tempDir, 0775, true)) {
        if (!is_dir($tempDir)){
            output_message("Failed to create temporary directory: {$tempDir}", true);
            exit(1);
        }
    }
}
if (!is_dir($finalDbDir)) {
    if (!@mkdir($finalDbDir, 0775, true)) {
        if (!is_dir($finalDbDir)){
            output_message("Failed to create final database directory: {$finalDbDir}", true);
            exit(1);
        }
    }
}


function download_ip2location_file($url, $postFields, $outputFilePath) {
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($ch, CURLOPT_TIMEOUT, 300);
    curl_setopt($ch, CURLOPT_USERAGENT, 'iskwaf2x MODX Component DB Downloader');
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postFields));
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_MAXREDIRS, 5);

    $fp = @fopen($outputFilePath, 'wb');
    if (!$fp) {
        output_message("Failed to open file for writing: {$outputFilePath}", true);
        curl_close($ch);
        return false;
    }
    curl_setopt($ch, CURLOPT_FILE, $fp);

    $curl_result = curl_exec($ch);
    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    $effectiveUrl = curl_getinfo($ch, CURLINFO_EFFECTIVE_URL);

    $curlErrorMessage = curl_error($ch);
    $curlErrorNumber = curl_errno($ch);

    curl_close($ch);
    fclose($fp);

    if ($curlErrorNumber) {
        output_message("cURL Error #{$curlErrorNumber} for {$outputFilePath} (final URL: {$effectiveUrl}): " . $curlErrorMessage, true);
        if (file_exists($outputFilePath)) @unlink($outputFilePath);
        return false;
    }

    if ($httpCode != 200) {
        output_message("HTTP Error {$httpCode} for {$outputFilePath} (final URL: {$effectiveUrl}). Check token, DB code, or IP2Location account status.", true);
        $errorContent = file_get_contents($outputFilePath);
        if (!empty($errorContent) && strlen($errorContent) < 2048) {
            output_message("Content of failed download attempt: " . substr(preg_replace('/\s+/', ' ', strip_tags($errorContent)), 0, 500) . "...", true);
        }
        if (file_exists($outputFilePath)) @unlink($outputFilePath);
        return false;
    }

    if (filesize($outputFilePath) == 0) {
        output_message("Downloaded file {$outputFilePath} is empty. This might indicate an issue with the download for product code " . ($postFields['file'] ?? 'N/A') . ".", true);
        if (file_exists($outputFilePath)) @unlink($outputFilePath);
        return false;
    }

    return true;
}


foreach ($databasesToDownload as $dbCode => $finalBinNameOnServer) {
    if (!isset($fileMappings[$dbCode])) {
        output_message("No file mapping information found for dbCode: {$dbCode}. Skipping.", true);
        continue;
    }

    $zipFileNameOnServer = $fileMappings[$dbCode]['zip_filename_on_server'];
    $binFileInZip = $fileMappings[$dbCode]['bin_in_zip'];
    $zipFilePath = $tempDir . $zipFileNameOnServer;

    output_message("Processing database code: {$dbCode} (ZIP: {$zipFileNameOnServer}, expecting BIN: {$binFileInZip}, final name: {$finalBinNameOnServer})");

    $downloadFields = [
        'token' => $token,
        'file'  => $dbCode,
    ];

    if (download_ip2location_file('https://www.ip2location.com/download', $downloadFields, $zipFilePath)) {
        output_message("Successfully downloaded {$zipFileNameOnServer}.");

        $zip = new ZipArchive;
        $res = $zip->open($zipFilePath);
        if ($res === TRUE) {
            output_message("Extracting {$binFileInZip} from {$zipFileNameOnServer} to {$tempDir}...");

            $extractedSuccessfully = false;
            for ($i = 0; $i < $zip->numFiles; $i++) {
                $entryNameInZip = $zip->getNameIndex($i);
                if (strtoupper(basename($entryNameInZip)) === strtoupper($binFileInZip)) {
                    if ($zip->extractTo($tempDir, $entryNameInZip)) {
                        $sourcePathAfterExtract = $tempDir . $entryNameInZip;
                        $tempBinPath = $tempDir . $binFileInZip;
                        if ($sourcePathAfterExtract !== $tempBinPath && file_exists($sourcePathAfterExtract) && !file_exists($tempBinPath)) {
                            if (!is_dir(dirname($tempBinPath))) @mkdir(dirname($tempBinPath), 0775, true);
                            rename($sourcePathAfterExtract, $tempBinPath);
                        }
                        if (file_exists($tempBinPath)) {
                            output_message("Successfully extracted {$binFileInZip} to {$tempBinPath}.");
                            $extractedSuccessfully = true;
                            break;
                        }
                    }
                }
            }
            $zip->close();

            if ($extractedSuccessfully) {
                $finalPath = $finalDbDir . $finalBinNameOnServer;
                if (rename($tempDir . $binFileInZip, $finalPath)) {
                    output_message("Moved {$binFileInZip} (as {$finalBinNameOnServer}) to {$finalPath}.");
                } else {
                    output_message("Failed to move {$tempDir}{$binFileInZip} to {$finalPath}.", true);
                }
            } else {
                output_message("Expected BIN file {$binFileInZip} not found or could not be extracted from {$zipFileNameOnServer}.", true);
            }
        } else {
            output_message("Failed to open ZIP archive: {$zipFilePath}. Error code: " . $res, true);
        }

        if (file_exists($zipFilePath)) {
            unlink($zipFilePath);
        }
    } else {
        output_message("Failed to download {$zipFileNameOnServer}.", true);
    }
    output_message("------------------------------------");
}


if (is_dir($tempDir)) {
    $filesInTemp = glob($tempDir . '*');
    foreach($filesInTemp as $file){
        if(is_file($file)) {
            @unlink($file);
        }
    }
    @rmdir($tempDir);
}

output_message("Database download process finished.");
exit(0);
