<?php
/**
 * IskWaf2xService - Сервисный класс для компонента iskwaf2x
 * Содержит всю логику WAF для MODX Revolution 2.x
 */

class IskWaf2xService
{
    /** @var modX */
    public $modx;

    /** @var array Конфигурация компонента */
    protected $config = array();

    /** @var array Данные текущего запроса */
    protected $requestData = array();

    /** @var xPDOObject|null Объект сработавшего правила */
    protected $matchedRule = null;

    /** @var string|null Конечное действие ('block', 'captcha', 'allow') */
    protected $finalAction = null;

    /** @var string|null Тип сработавшего правила ('ip', 'ua', 'ref', 'uri') */
    protected $matchedRuleType = '';

    /** @var string|null Идентификатор (паттерн) сработавшего правила */
    protected $matchedRuleIdentifier = '';

    /**
     * Конструктор класса
     * @param modX $modx
     */
    public function __construct(modX $modx)
    {
        $this->modx = $modx;

        if (session_status() == PHP_SESSION_NONE) {
            session_start();
        }

        $this->loadConfig();
        $this->loadRequestData();
        $this->loadModel();
    }

    /**
     * Загружает настройки компонента
     */
    protected function loadConfig()
    {
        $this->config['logSetting'] = $this->modx->getOption('iskwaf2x_iskwaf2x_option_log', null, 'triggered', true);
        $this->config['wafCheckMode'] = $this->modx->getOption('iskwaf2x_iskwaf2x_waf_check_mode', null, 'on_every_request', true);
        $this->config['captchaResourceId'] = (int) $this->modx->getOption('iskwaf2x_iskwaf2x_captcha_resource_id', null, 0);
    }

    /**
     * Собирает данные о текущем запросе
     */
    protected function loadRequestData()
    {
        $this->requestData = array(
            'ip' => isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : '0.0.0.0',
            'userId' => ($this->modx->user && $this->modx->user->isAuthenticated($this->modx->context->key)) ? $this->modx->user->get('id') : 0,
            'method' => isset($_SERVER['REQUEST_METHOD']) ? $_SERVER['REQUEST_METHOD'] : 'UNKNOWN',
            'uri' => isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : '/',
            'userAgent' => isset($_SERVER['HTTP_USER_AGENT']) ? (string)$_SERVER['HTTP_USER_AGENT'] : '',
            'referrer' => isset($_SERVER['HTTP_REFERER']) ? (string)$_SERVER['HTTP_REFERER'] : '',
            'timestamp' => time(),
        );
    }

    /**
     * Подключает xPDO модель компонента
     */
    protected function loadModel()
    {
        $modelPath = $this->modx->getOption('core_path', null, MODX_CORE_PATH) . 'components/iskwaf2x/model/';
        $added = $this->modx->addPackage('iskwaf2x', $modelPath);

        if (!$added) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[iskwaf2x] Could not load iskwaf2x model package from: ' . $modelPath);
        }
    }

    /**
     * Главный исполняемый метод
     */
    public function run()
    {
        $skipWafRules = false;
        if ($this->config['wafCheckMode'] === 'once_per_session' && !empty($_SESSION['iskwaf2x_session_cleared'])) {
            $skipWafRules = true;
        }

        if (!$skipWafRules) {
            $this->checkAllRules();
        }

        if ($this->matchedRule) {
            $this->performAction();
        } else {
            $this->logCleanVisit($skipWafRules);
        }
    }

    /**
     * Последовательно выполняет все проверки правил
     */
    protected function checkAllRules()
    {
        if (!$this->checkIpRules()) {
            if (!$this->checkUaRules()) {
                if (!$this->checkRefRules()) {
                    $this->checkUriRules();
                }
            }
        }
    }

    /**
     * Сохраняет детали сработавшего правила в свойствах класса
     */
    protected function setMatchedRuleDetails($ruleObject, $ruleType, $identifierKey)
    {
        $this->matchedRule = $ruleObject;
        $this->finalAction = $ruleObject->get('rule_type');
        $this->matchedRuleType = $ruleType;
        $this->matchedRuleIdentifier = $ruleObject->get($identifierKey);
    }

    protected function checkIpRules()
    {
        $q = $this->modx->newQuery('IskWafRuleIp');
        $q->where(array('active' => 1, 'ip_cidr' => $this->requestData['ip']));
        $rule = $this->modx->getObject('IskWafRuleIp', $q);

        if ($rule) {
            $this->setMatchedRuleDetails($rule, 'ip', 'ip_cidr');
            return true;
        }

        $q = $this->modx->newQuery('IskWafRuleIp');
        $q->where(array('active' => 1, 'ip_cidr:LIKE' => '%/%'));
        $rules = $this->modx->getIterator('IskWafRuleIp', $q);
        $bestMatch = null;
        $narrowestMask = -1;

        foreach ($rules as $rule) {
            $ruleCidr = $rule->get('ip_cidr');
            if (self::ip_in_cidr($this->requestData['ip'], $ruleCidr)) {
                list(, $maskStr) = explode('/', $ruleCidr, 2);
                $currentMask = (int)$maskStr;
                if ($currentMask > $narrowestMask) {
                    $narrowestMask = $currentMask;
                    $bestMatch = $rule;
                }
            }
        }

        if ($bestMatch) {
            $this->setMatchedRuleDetails($bestMatch, 'ip', 'ip_cidr');
            return true;
        }

        return false;
    }

    protected function checkUaRules()
    {
        $q = $this->modx->newQuery('IskWafRuleUserAgent');
        $q->where(array('active' => 1));
        $rules = $this->modx->getIterator('IskWafRuleUserAgent', $q);

        foreach ($rules as $rule) {
            $pattern = $rule->get('user_agent_pattern');
            $type = $rule->get('pattern_type');
            $matched = false;
            switch ($type) {
                case 'exact': $matched = (strtolower($this->requestData['userAgent']) === strtolower($pattern)); break;
                case 'contains': $matched = (stripos($this->requestData['userAgent'], $pattern) !== false); break;
                case 'regex': $matched = (@preg_match($pattern, $this->requestData['userAgent'])); break;
            }
            if ($matched) {
                $this->setMatchedRuleDetails($rule, 'ua', 'user_agent_pattern');
                return true;
            }
        }
        return false;
    }

    protected function checkRefRules()
    {
        $q = $this->modx->newQuery('IskWafRuleReferrer');
        $q->where(array('active' => 1));
        $rules = $this->modx->getIterator('IskWafRuleReferrer', $q);

        foreach ($rules as $rule) {
            $pattern = $rule->get('referrer_pattern');
            $type = $rule->get('pattern_type');
            $matched = false;
            switch ($type) {
                case 'exact': $matched = (strtolower($this->requestData['referrer']) === strtolower($pattern)); break;
                case 'contains': $matched = (!empty($pattern) && stripos($this->requestData['referrer'], $pattern) !== false); break;
                case 'regex': $matched = (@preg_match($pattern, $this->requestData['referrer'])); break;
            }
            if ($matched) {
                $this->setMatchedRuleDetails($rule, 'ref', 'referrer_pattern');
                return true;
            }
        }
        return false;
    }

    protected function checkUriRules()
    {
        $q = $this->modx->newQuery('IskWafRuleRequestUri');
        $q->where(array('active' => 1));
        $rules = $this->modx->getIterator('IskWafRuleRequestUri', $q);

        foreach ($rules as $rule) {
            $pattern = $rule->get('uri_pattern');
            $type = $rule->get('pattern_type');
            $uriToMatch = $rule->get('match_query_string') ? $this->requestData['uri'] : explode('?', $this->requestData['uri'], 2)[0];
            $matched = false;
            switch ($type) {
                case 'exact': $matched = (strtolower($uriToMatch) === strtolower($pattern)); break;
                case 'contains': $matched = (stripos($uriToMatch, $pattern) !== false); break;
                case 'regex': $matched = (@preg_match($pattern, $uriToMatch)); break;
                case 'starts_with': $matched = (strpos(strtolower($uriToMatch), strtolower($pattern)) === 0); break;
            }
            if ($matched) {
                $this->setMatchedRuleDetails($rule, 'uri', 'uri_pattern');
                return true;
            }
        }
        return false;
    }

    /**
     * Выполняет действие и логирование
     */
    protected function performAction()
    {
        if ($this->config['logSetting'] !== 'off') {
            $this->logTriggeredRule();
        }

        switch ($this->finalAction) {
            case 'block':
                header('HTTP/1.1 403 Forbidden');
                echo '<!DOCTYPE html><html><head><title>Forbidden</title></head><body><h1>403 Forbidden</h1><p>Access denied.</p></body></html>';
                @session_write_close();
                exit();

            case 'captcha':
                $this->handleCaptcha();
                break;

            case 'allow':
                if ($this->config['wafCheckMode'] === 'once_per_session') {
                    $_SESSION['iskwaf2x_session_cleared'] = true;
                }
                if (isset($_SESSION['iskwaf2x_captcha_active_flow'])) {
                    unset($_SESSION['iskwaf2x_captcha_active_flow'], $_SESSION['iskwaf2x_captcha_original_uri'], $_SESSION['iskwaf2x_expected_captcha'], $_SESSION['iskwaf2x_full_captcha_code'], $_SESSION['iskwaf2x_captcha_task_type']);
                }
                break;
        }
    }

    /**
     * Обрабатывает логику CAPTCHA
     */
    protected function handleCaptcha()
    {
        if (!empty($_SESSION['iskwaf2x_captcha_passed'])) {
            return;
        }
        if ($this->requestData['method'] === 'POST' && isset($_POST['iskwaf2x_captcha_code'])) {
            $this->modx->lexicon->load('iskwaf2x:default');
            $isCorrect = !empty($_SESSION['iskwaf2x_expected_captcha']) && strtolower(trim($_POST['iskwaf2x_captcha_code'])) === strtolower($_SESSION['iskwaf2x_expected_captcha']);

            unset($_SESSION['iskwaf2x_expected_captcha'], $_SESSION['iskwaf2x_full_captcha_code'], $_SESSION['iskwaf2x_captcha_task_type']);

            if ($isCorrect) {
                $_SESSION['iskwaf2x_captcha_passed'] = true;
                if ($this->config['wafCheckMode'] === 'once_per_session') {
                    $_SESSION['iskwaf2x_session_cleared'] = true;
                }
                $redirectTo = !empty($_SESSION['iskwaf2x_captcha_original_uri']) ? $_SESSION['iskwaf2x_captcha_original_uri'] : $this->modx->getOption('site_url');
                unset($_SESSION['iskwaf2x_captcha_original_uri'], $_SESSION['iskwaf2x_captcha_active_flow']);
                @session_write_close();
                $this->modx->sendRedirect($redirectTo);
                exit();
            } else {
                $_SESSION['iskwaf2x_captcha_error_message'] = $this->modx->lexicon('iskwaf2x_captcha_err_invalid');
                if ($this->config['captchaResourceId'] > 0) {
                    $url = $this->modx->makeUrl($this->config['captchaResourceId'], '', '', 'full');
                    if (!empty($url)) {
                        @session_write_close();
                        $this->modx->sendRedirect($url);
                    } else {
                        $this->modx->log(modX::LOG_LEVEL_ERROR, '[iskwaf2x] CAPTCHA_FAIL: Could not generate redirect URL for CAPTCHA page ID: ' . $this->config['captchaResourceId']);
                        header('HTTP/1.1 503 Service Unavailable'); echo 'CAPTCHA page redirect error.'; @session_write_close(); die();
                    }
                } else {
                    $this->modx->log(modX::LOG_LEVEL_ERROR, '[iskwaf2x] CRITICAL: CAPTCHA resource ID not set.');
                    header('HTTP/1.1 503 Service Unavailable'); echo 'CAPTCHA system error (redirect).'; @session_write_close(); die();
                }
                exit();
            }
        }
        if (empty($_SESSION['iskwaf2x_captcha_active_flow'])) {
            $_SESSION['iskwaf2x_captcha_original_uri'] = $this->requestData['uri'];
            $_SESSION['iskwaf2x_captcha_active_flow'] = true;
        }
        if ($this->config['captchaResourceId'] > 0) {
            $this->modx->sendForward($this->config['captchaResourceId']);
        } else {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[iskwaf2x] CRITICAL: CAPTCHA resource ID not set.');
            header('HTTP/1.1 503 Service Unavailable'); echo 'CAPTCHA system error.'; @session_write_close(); die();
        }
    }

    /**
     * Логирует сработавшее правило
     */
    protected function logTriggeredRule()
    {
        $logEntry = $this->modx->newObject('IskWafLog');
        if (!$logEntry) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[iskwaf2x] Could not create IskWafLog object.');
            return;
        }

        $notes = 'Rule type: ' . $this->matchedRuleType . '. Matched: ' . mb_substr($this->matchedRuleIdentifier, 0, 220);
        if ($this->matchedRuleType === 'uri' && $this->matchedRule instanceof IskWafRuleRequestUri) {
            $notes .= '. MatchQuery: ' . ($this->matchedRule->get('match_query_string') ? 'Yes' : 'No');
        }

        $logData = array(
            'timestamp' => date('Y-m-d H:i:s', $this->requestData['timestamp']),
            'ip_address' => $this->requestData['ip'],
            'user_id' => $this->requestData['userId'],
            'method' => $this->requestData['method'],
            'request_uri' => $this->requestData['uri'],
            'user_agent' => mb_substr($this->requestData['userAgent'], 0, 1000),
            'referrer' => mb_substr($this->requestData['referrer'], 0, 1000),
            'waf_action' => $this->finalAction,
            'triggered_rule' => $this->matchedRuleType . ':' . $this->matchedRule->get('id'),
            'notes' => $notes,
            'http_status' => ($this->finalAction === 'block' ? 403 : 200)
        );
        $logEntry->fromArray($logData);

        if ($this->requestData['method'] === 'POST') {
            $logEntry->set('request_body_preview', mb_substr(file_get_contents('php://input'), 0, 4096));
        }

        if (!$logEntry->save()) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[iskwaf2x] Failed to save triggered log entry.');
        }
    }

    /**
     * Логирует "чистый" визит
     */
    protected function logCleanVisit($skippedBySession = false)
    {
        if ($this->config['logSetting'] === 'off') return;

        $logAction = 'VISIT';
        $logNotes = 'No WAF rules triggered.';

        if ($skippedBySession) {
            $logAction = 'VISIT_SESSION_CLEARED';
            $logNotes = 'WAF rules skipped (once_per_session mode, session previously cleared).';
        } elseif ($this->config['wafCheckMode'] === 'once_per_session') {
            $_SESSION['iskwaf2x_session_cleared'] = true;
        }

        if ($this->config['logSetting'] !== 'full' && !$skippedBySession) {
            return;
        }

        $logEntry = $this->modx->newObject('IskWafLog');
        if (!$logEntry) { return; }

        $logData = array(
            'timestamp' => date('Y-m-d H:i:s', $this->requestData['timestamp']),
            'ip_address' => $this->requestData['ip'],
            'user_id' => $this->requestData['userId'],
            'method' => $this->requestData['method'],
            'request_uri' => $this->requestData['uri'],
            'user_agent' => mb_substr($this->requestData['userAgent'], 0, 1000),
            'referrer' => mb_substr($this->requestData['referrer'], 0, 1000),
            'waf_action' => $logAction,
            'notes' => $logNotes
        );
        $logEntry->fromArray($logData);

        if ($this->requestData['method'] === 'POST') {
            $logEntry->set('request_body_preview', mb_substr(file_get_contents('php://input'), 0, 4096));
        }

        if (!$logEntry->save()) {
            $this->modx->log(modX::LOG_LEVEL_ERROR, '[iskwaf2x] Failed to save clean visit log entry.');
        }
    }

    /**
     * Проверяет, входит ли IP-адрес в CIDR-диапазон.
     */
    private static function ip_in_cidr($ip, $cidr)
    {
        if (strpos($cidr, '/') === false) {
            return ($ip === $cidr);
        }
        list($subnet, $mask) = explode('/', $cidr, 2);
        if (!filter_var($ip, FILTER_VALIDATE_IP) || !filter_var($subnet, FILTER_VALIDATE_IP) || !ctype_digit($mask)) {
            return false;
        }
        $ip_protocol = (strpos($ip, ':') === false) ? 4 : 6;
        $subnet_protocol = (strpos($subnet, ':') === false) ? 4 : 6;
        if ($ip_protocol !== $subnet_protocol) {
            return false;
        }
        if ($ip_protocol === 4) {
            if ($mask < 0 || $mask > 32) return false;
            $ip_long = ip2long($ip);
            $subnet_long = ip2long($subnet);
            if ($ip_long === false || $subnet_long === false) return false;
            $mask_long = -1 << (32 - $mask);
            return ($ip_long & $mask_long) === ($subnet_long & $mask_long);
        } else {
            if ($mask < 0 || $mask > 128) return false;
            $ip_bin = inet_pton($ip);
            $subnet_bin = inet_pton($subnet);
            if ($ip_bin === false || $subnet_bin === false) return false;
            $mask_bin = '';
            $mask_bytes = floor($mask / 8);
            $mask_bits = $mask % 8;
            for ($i = 0; $i < $mask_bytes; $i++) { $mask_bin .= chr(255); }
            if ($mask_bits > 0) { $mask_bin .= chr(255 << (8 - $mask_bits)); $mask_bytes++; }
            while ($mask_bytes < 16) { $mask_bin .= chr(0); $mask_bytes++; }
            return ($ip_bin & $mask_bin) === ($subnet_bin & $mask_bin);
        }
    }
}